import pandas as pd
from pysentimentanalyzer.get_aggregated_sentiment_score import aggregate_sentiment_score

def convert_to_likert(df, col):
    """Convert the sentiment scores to a likert scale from 1-5
    
    Parameters
    ----------
    df : Pandas DataFrame
        DataFrame containing the text to perform sentiment analysis
    col : string
        Name of the column of text in the Dataframe

    Returns
    -------
    string:
        description of the likert scale value
    int:
        likert scale value
        
    Examples
    --------
    >>> convert_to_likert(counts)
    """
    if not isinstance(df, pd.DataFrame):
        raise Exception("The first parameter should be a Pandas DataFrame.")
    if not (col in list(df.columns)):
        raise Exception("The column parameter should be a column in the DataFrame.")
    agg_score = aggregate_sentiment_score(df, col)
    if agg_score >= -1 and agg_score < -0.6:
        return "very negative", 1
    elif agg_score >= -0.6 and agg_score < -0.2:
        return "negative", 2
    elif agg_score >= -0.2 and agg_score <= 0.2:
        return "neutral", 3
    elif agg_score > 0.2 and agg_score <= 0.6:
        return "positive", 4
    elif agg_score >0.6 and agg_score <= 1:
        return "very positive", 5
    else:
        raise Exception("Aggregate compound score not in the valid range.")
    


import pandas as pd
from wordcloud import WordCloud
from pysentimentanalyzer.helper import get_sentiment_and_score

def generate_wordcloud(df, col):
    """
    Generate the word cloud of the most common positive and negative words from a given survey in the form of a 
    data frame and create a word cloud.

    Parameters
    ----------
    df : Pandas DataFrame
        DataFrame containing the survey comments
    col : string
        Name of the column of text in the Dataframe

    Returns
    -------
    List:
         A list of three images in the following order: postive, neutral, negative
         A word cloud instance that can be later plotted or saved as an image

    Examples
    --------
     >>> imgs= generate_wordcloud(df, col)
     >>> imgs[0].show() # positive
     >>> imgs[1].show() # negative
     >>> imgs[2].show() # neutral
    """
    if df is None:
        raise TypeError("df must not be None")
    
    if not isinstance(df, pd.DataFrame):
        raise TypeError("df must be a DataFrame")
    
    if col is None:
        raise TypeError("col must not be None") 

    df =  get_sentiment_and_score(df,col) 
    neg_df = df[df["sentiment"] == "negative"]
    pos_df = df[df["sentiment"] == "positive"]
    neutral_df = df[df["sentiment"] == "neutral"]

    negative_messages = " ".join(neg_df[col].tolist())
    positive_messages = " ".join(pos_df[col].tolist())
    neutral_messages = " ".join(neutral_df[col].tolist())


    wordcloud = WordCloud(max_font_size=40, width=800, height=400)
    pos_image = wordcloud.generate(negative_messages).to_image()
    neg_image = wordcloud.generate(positive_messages).to_image()
    neutral_image = wordcloud.generate(neutral_messages).to_image()

    return [pos_image,neutral_image, neg_image]


import nltk
import pandas as pd
from nltk.sentiment.vader import SentimentIntensityAnalyzer

nltk.download("vader_lexicon")
nltk.download("punkt")

def get_compound_score(text):
    """
    Calculates a compound sentiment score from text

    Parameters:
    -----------
    text: str
        Input text data as string
    
    Returns:
    --------
    A numeric sentiment score of the text : (float)
    """
    sid = SentimentIntensityAnalyzer()
    score = sid.polarity_scores(text)
    return score["compound"]

def get_sentiment(text):
    """
    Evaluates the type of sentiment from a given text

    Parameters:
    -----------
    text: str
        Input text data as string
    
    Returns:
    --------
    The type of sentiment (positive, negetive or neutral) as a string : (str)
    """
    score = get_compound_score(text)
    if score > 0:
        return "positive"
    if score < 0:
        return "negative"
    return "neutral"

def get_sentiment_and_score(df, col):
    """
    Calculates a compound sentiment score from text

    Parameters:
    -----------
    df: pd.DataFrame
        A DataFrame
    col: 
        The column name of the text column on which sentiment analysis is done
    
    Returns:
    --------
    A Pandas DataFrame with numeric sentiment and its corresponding sentiment type 
    added to the DataFrame : (pd.DataFrame)
    """
    df = df.assign(compound_score=df[col].apply(get_compound_score))
    df = df.assign(sentiment=df[col].apply(get_sentiment))
    return df
